/**
 * async 格式化包装
 * @param { promise } promise 被包装promise
 * @returns { pormise } 包装后的promise
 * @summary 将 aysnce await 错误作为返回值的处理方式
 *
 * @example
 *  asynce load(){
 *
 *      const [ res, err ] = await asyncFormat( api(...) )
 *
 *      if(res === null){
 *        console.log(err)
 *        return
 *      }
 *
 *  }
 */
async function asyncFormat(promise) {
    return promise.then(res => [res, null]).catch((err) => [null, err]);
}

/**
 * 拼接路由参数
 * @param { object } obj 参数对象
 * @returns { string }
 */
function joinUrlParams(url, obj) {
    if (!obj) {
        return url;
    }
    return Object.entries({ ...obj }).reduce((acc, [key, value], index) => `${acc}${index ? '&' : '?'}${key}=${value}`, url);
}

/**
 * 对象过滤器
 * @summary
 * 通过关键字列表，抽取对象对应字段值，生成新的对象.
 * 只做浅层映射
 * @example
 * const source = {
 *   id: 12,
 *   name: 'coco',
 *   job: 'IT'
 * }
 * const keys = ['name', 'job']
 * const target = objectFilter(keys, source)
 * => { name: 'coco', job: 'IT' }
 */
function objectFilter(keys, source) {
    return keys.reduce((acc, next) => {
        return {
            ...acc,
            [next]: source[next] || undefined
        };
    }, {});
}

/**
 * 延时器
 * @author copy-left
 * @summary
 * 通过promise 将函数推入任务队列，延时执行
 */
function delay(time, cb) {
    return new Promise((reslove) => {
        setTimeout(() => {
            reslove(cb ? cb() : null);
        }, time * 1000);
    });
}

/**
 * 获取对象类型字符
 * @param data
 * @returns 类型字符
 */
function type(data) {
    const type = Object.prototype.toString.call(data);
    return type.replace(/\[object|\]|\s/igm, '').toLocaleLowerCase();
}
/**
 * 类型校验
 * @param data
 * @param typeStr
 */
function isType(data, typeStr) {
    return type(data) === typeStr;
}
// 基础类型
const isString = (data) => type(data) === 'string';
const isNumber = (data) => type(data) === 'number';
const isBoolean = (data) => type(data) === 'boolean';
const isUndefined = (data) => data === undefined;
const isNull = (data) => data === null;
const isArray = (data) => type(data) === 'array';
const isFunction = (data) => type(data) === 'function';
const isObject = (data) => type(data) === 'object';
// 特定类型
const isEventName = (data) => (/^on[A-Z]+[a-z]+$/).test(data);
const isSlotName = (data) => (/^v-slot(\:{1}[a-zA-Z]+)?$/).test(data);

/**
 * 本地缓存包装函数
 * @author copy-left
 * @time 2020-11-10
 *
 */
let registed = false;
/**
 * 本地缓存包装函数
 *
 * @summary
 * 为本地缓存对象 localStorage，sessionStorage 添加 setItem getItem 包装函数
 * 包装函数将原字符参数或返回值通过JSON转换为对象
 *
 * @remind
 * 因为使用JSON对数据做转换, 数据格式必须服务json格式, 所以报错纯字符时将报错,
 * 此时可以使用原生方法
 *
 * @example
 * import { registSaveFn } from '@micro/utils'
 * registSaveFn() //全局调用一次
 *
 * const saveKey = 'USER'
 * localStorage.$setItem(saveKey, { name: 'copy', sex: 'man' })
 * const user = localStorage.$getItem(saveKey)
 * console.log(user.name)
 */
function registSaveFn() {
    if (registed) {
        return;
    }
    localStorage.__proto__.$setItem = buildSetItem(localStorage);
    localStorage.__proto__.$getItem = buildGetItem(localStorage);
    sessionStorage.__proto__.$setItem = buildSetItem(sessionStorage);
    sessionStorage.__proto__.$getItem = buildGetItem(sessionStorage);
    registed = true;
}
function buildSetItem(storage) {
    return (key, data) => {
        try {
            const dataStr = JSON.stringify(data);
            storage.setItem(key, dataStr);
        }
        catch (error) {
            console.error(`${typeof (storage)} 设置失败: `, error);
            return null;
        }
    };
}
function buildGetItem(storage) {
    return (key) => {
        const dataStr = storage.getItem(key);
        try {
            return dataStr ? JSON.parse(dataStr) : null;
        }
        catch (error) {
            console.error(`${typeof (storage)} 获取失败: `, error);
            return null;
        }
    };
}

export { asyncFormat, buildGetItem, buildSetItem, delay, isArray, isBoolean, isEventName, isFunction, isNull, isNumber, isObject, isSlotName, isString, isType, isUndefined, joinUrlParams, objectFilter, registSaveFn, type };
